// $Id: IPEncoding.java,v 1.5 2001/06/05 07:16:23 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;


/**
 * IP protocol encoding information.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.5 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/06/05 07:16:23 $
 */
public class IPEncoding
{
  /**
   * IPPacket factory.
   */
  public static IPPacket create(int lLen, byte [] bytes) {
    String msa = EthernetEncoding.extractMACAddress(lLen, true, bytes); 
    String mda = EthernetEncoding.extractMACAddress(lLen, false, bytes); 

    String srcAddress = IPEncoding.extractAddress(true, lLen, bytes);
    String dstAddress = IPEncoding.extractAddress(false, lLen, bytes);

    int protocol = IPEncoding.extractProtocol(lLen, bytes);
    return new IPPacket(lLen, bytes, msa, mda, srcAddress, dstAddress,
                        protocol);
  }

  /**
   * Extract the protocol code from packet data. The packet data 
   * must contain an IP datagram.
   * The protocol code specifies what kind of information is contained in the 
   * data block of the ip datagram.
   * @param llen the length of the link-level header.
   * @param packetBytes packet bytes, including the link-layer header.
   * @return the IP protocol code. i.e. 0x06 signifies TCP protocol.
   */
  public static int extractProtocol(int lLen, byte [] packetBytes) {
    return packetBytes[lLen + CODE_POS];
  }

  /**
   * Extract an IP address from an IP packet.
   * @param offset the offset of the address data from the start of the 
   * packet.
   * @param packetBytes packet bytes, including the link-layer header.
   */
  public static String extractAddress(boolean sourceFlag, 
                                      int lLen, 
                                      byte [] packetBytes) {
    int offset = sourceFlag ? lLen + SRC_POS : lLen + DST_POS;
    return IPAddress.extract(offset, packetBytes);
  }


  // todo: the following can change, and potentially affect headers
  // offset by the length of the IP header, if there are option fields 
  // in the ip header.

  /**
   * Position of the IP protocol code within a packet.
   */
  public final static int CODE_POS = 9;

  /**
   * Position of the source IP address within a packet.
   */
  public final static int SRC_POS = 12;

  /**
   * Position of the destination IP address within a packet.
   */
  public final static int DST_POS = SRC_POS + IPAddress.WIDTH;

  /**
   * Length in bytes of an IP header.
   */
  public final static int HEADER_LEN = 20;


  private String _rcsid = 
    "$Id: IPEncoding.java,v 1.5 2001/06/05 07:16:23 pcharles Exp $";
}
